/*
 -------------------------------------------------------------------------------
 | Producer - Copyright (C) 2001-2007  Don Burns                               |
 |                                                                             |
 | This library is free software; you can redistribute it and/or modify        |
 | it under the terms of the GNU Lesser General Public License as published    |
 | by the Free Software Foundation; either version 3 of the License, or        |
 | (at your option) any later version.                                         |
 |                                                                             |
 | This library is distributed in the hope that it will be useful, but         |
 | WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  |
 | or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public     |
 | License for more details.                                                   |
 |                                                                             |
 | You should have received a copy of the GNU Lesser General Public License    |
 | along with this software; if not, write to the Free Software Foundation,    |
 | Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.               |
 |                                                                             |
 -------------------------------------------------------------------------------
 */


#ifndef PRODUCER_EXAMPLE_KEYBOARD_MOUSE_CALLBACK_DEF
#define PRODUCER_EXAMPLE_KEYBOARD_MOUSE_CALLBACK_DEF 1

#include <Producer/KeyboardMouse.h>

class KeyboardMouseResponder : public Producer::KeyboardMouseResponder
{
    public:
		KeyboardMouseResponder(bool &done) :
			_mx(0.0f),_my(0.0f),_mbutton(0),
			_done(done),
            _resetTrackball(false),
            _toggleLighting(false),
            _toggleTexturing(false),
            _toggleWireframe(false),
            _toggleBackfaceCulling(false)
			{}

		virtual bool specialKeyPress( Producer::KeyCharacter key )
		{
			switch( key )
			{
				case Producer::KeyChar_Escape:
					_done = true;
					break;

                default:
                    break;
			}
            return false;
		}

        virtual bool shutdown()
        {
            _done = true; 
            return true;
        }

		virtual bool keyPress( Producer::KeyCharacter key )
		{
            switch( key )
            {
                case Producer::KeyChar_b:
                    _toggleBackfaceCulling = true;
                    break;

                case Producer::KeyChar_l:
                    _toggleLighting = true;
                    break;

                case Producer::KeyChar_t:
                    _toggleTexturing = true;
                    break;

                case Producer::KeyChar_w:
                    _toggleWireframe = true;
                    break;

                case Producer::KeyChar_space:
                    _resetTrackball = true;
                    break;

                default:
                    break;
            }

            return false;
		}

		virtual bool mouseMotion( float mx, float my ) 
		{
			_mx = mx;
			_my = my;
            return false;
		}
		virtual bool buttonPress( float mx, float my, unsigned int mbutton ) 
		{
			_mx = mx;
			_my = my;
			_mbutton |= (1<<(mbutton-1));
            return false;
		}
		virtual bool buttonRelease( float mx, float my, unsigned int mbutton ) 
		{
			_mx = mx;
			_my = my;
			_mbutton &= ~(1<<(mbutton-1));
            return false;
		}

		bool done() { return _done; }
		float mx()  { return _mx; }
		float my()  { return _my; }
		unsigned int mbutton()  { return _mbutton; }

        bool resetTrackball() { return _checkAndResetToggle( _resetTrackball ); }
        bool toggleLighting() { return _checkAndResetToggle( _toggleLighting ); }
        bool toggleTexturing() { return _checkAndResetToggle( _toggleTexturing ); }
        bool toggleWireframe() { return _checkAndResetToggle( _toggleWireframe ); }
        bool toggleBackfaceCulling() { return _checkAndResetToggle( _toggleBackfaceCulling ); }

    protected:
		virtual ~KeyboardMouseResponder() {}

    private:
		float _mx, _my;
		unsigned int _mbutton;
		bool &_done;
        bool _resetTrackball;
        bool _toggleLighting;
        bool _toggleTexturing;
        bool _toggleWireframe;
        bool _toggleBackfaceCulling;

        bool _checkAndResetToggle( bool &toggle )
        {
            return toggle == true ? !(toggle=false) : false;
        }

};

#endif
